==> webservices-rt-2.3-sources.jar: com/sun/xml/ws/rx/rm/runtime/sequence/SequenceManagerFactory.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.xml.ws.rx.rm.runtime.sequence;

import com.sun.xml.ws.api.server.WSEndpoint;
import com.sun.xml.ws.commons.WSEndpointCollectionBasedMOMListener;
import com.sun.xml.ws.rx.rm.runtime.RmConfiguration;
import com.sun.xml.ws.rx.rm.runtime.delivery.DeliveryQueueBuilder;
import com.sun.xml.ws.rx.rm.runtime.sequence.invm.InVmSequenceManager;
import com.sun.xml.ws.rx.rm.runtime.sequence.persistent.PersistentSequenceManager;
import com.sun.xml.ws.server.WSEndpointImpl;
import com.sun.xml.ws.server.WSEndpointMOMProxy;
import org.glassfish.gmbal.ManagedObjectManager;

import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;

/**
 *
 * @author Marek Potociar (marek.potociar at sun.com)
 */
public enum SequenceManagerFactory {
    INSTANCE;

    private final WSEndpointCollectionBasedMOMListener listener;
    private final WeakHashMap<WSEndpoint, SequenceManager> sequenceManagersForDeferredRegistration = new WeakHashMap<WSEndpoint, SequenceManager>();
        
    private SequenceManagerFactory() {
        // TODO: load from external configuration and revert to default if not present

        listener = new WSEndpointCollectionBasedMOMListener(this, SequenceManager.MANAGED_BEAN_NAME, sequenceManagersForDeferredRegistration);
        listener.initialize();
    }

    /**
     * Creates new {@link SequenceManager} instance. This operation should be called only once per endpoint and/or endpoint client.
     *
     * @param persistent specifies whether returned {@link SequenceManager} instance should support persistent message storage
     * @param uniqueEndpointId unique identifier of the WS endpoint for which this particular sequence manager will be used. The endpoint
     * identifier must be different for the client and for the server side.
     * @param inboundQueueBuilder delivery queue builder that will be used to create delivery queue for all newly created inbound sequences
     * @param outboundQueueBuilder delivery queue builder that will be used to create delivery queue for all newly created outbound sequences
     * @param managedObjectManager object manager managing the newly created {@link SequenceManager} instance
     * @return newly created {@link SequenceManager} instance
     */
    public SequenceManager createSequenceManager(boolean persistent, String uniqueEndpointId, DeliveryQueueBuilder inboundQueueBuilder, DeliveryQueueBuilder outboundQueueBuilder, RmConfiguration configuration) {
        synchronized (INSTANCE) {
            SequenceManager result;
            if (persistent) {
                result = new PersistentSequenceManager(uniqueEndpointId, inboundQueueBuilder, outboundQueueBuilder, configuration);
            } else {
                result = new InVmSequenceManager(uniqueEndpointId, inboundQueueBuilder, outboundQueueBuilder, configuration);
            }
            
            ManagedObjectManager mom = configuration.getManagedObjectManager();
            handleMOMRegistration(result, mom, true);

            return result;
        }
    }
    
    /**
     * Disposes the sequence manager properly and unregisters it from the ManagedOnjectManager
     * 
     * @param manager {@link SequenceManager} instance to be disposed
     */
    public void dispose(SequenceManager manager, RmConfiguration configuration) {
        synchronized (INSTANCE) {
            manager.dispose();

            ManagedObjectManager mom = configuration.getManagedObjectManager();
            handleMOMRegistration(manager, mom, false);
        }
    }

    /**
     * Handles (un)registration process of {@code SequenceManager} in {@link ManagedObjectManager}.
     * 
     * @param manager SequenceManager to be (un)registered
     * @param managedObjectManager ManagedObjectManager instance where SequenceManager will be registered at
     * @param register {@code true} if the manager should be registered, {@code false} for unregistration
     */
    private void handleMOMRegistration(final SequenceManager manager, final ManagedObjectManager managedObjectManager, final boolean register) {
        if (manager == null || managedObjectManager == null) {
            return;
        }

        if (!listener.canRegisterAtMOM() && (managedObjectManager instanceof WSEndpointMOMProxy)) {
            // SequenceManager cannot be (un)registered directly so postpone its (un)registration until JMX connection
            // is created
            final WSEndpointMOMProxy endpointMOMProxy = (WSEndpointMOMProxy) managedObjectManager;
            final WSEndpointImpl wsEndpoint = endpointMOMProxy.getWsEndpoint();

            if (register) {
                sequenceManagersForDeferredRegistration.put(wsEndpoint, manager);
            } else {
                sequenceManagersForDeferredRegistration.remove(wsEndpoint);
            }
        } else {
            // 1) SequenceManager can be (un)registered directly - process its (un)registration
            // 2) (not expected) SequenceManager cannot be (un)registered directly but also it cannot be postponed hence
            //    the (un)registration is going to be forced (managedObjectManager (un)registration as well)

            if (register) {
                listener.registerAtMOM(manager, managedObjectManager);
            } else {
                listener.unregisterFromMOM(manager, managedObjectManager);
            }
        }
    }
    
}

==> webservices-rt-2.3.1-sources.jar: com/sun/xml/ws/rx/rm/runtime/sequence/SequenceManagerFactory.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 1997-2013 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.xml.ws.rx.rm.runtime.sequence;

import com.sun.xml.ws.api.server.Container;
import com.sun.xml.ws.api.server.WSEndpoint;
import com.sun.xml.ws.commons.WSEndpointCollectionBasedMOMListener;
import com.sun.xml.ws.rx.rm.runtime.LocalIDManager;
import com.sun.xml.ws.rx.rm.runtime.RmConfiguration;
import com.sun.xml.ws.rx.rm.runtime.delivery.DeliveryQueueBuilder;
import com.sun.xml.ws.rx.rm.runtime.sequence.invm.InVmSequenceManager;
import com.sun.xml.ws.rx.rm.runtime.sequence.persistent.PersistentSequenceManager;
import com.sun.xml.ws.server.WSEndpointImpl;
import com.sun.xml.ws.server.WSEndpointMOMProxy;
import org.glassfish.gmbal.ManagedObjectManager;

import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;

/**
 *
 * @author Marek Potociar (marek.potociar at sun.com)
 */
public enum SequenceManagerFactory {
    INSTANCE;

    private final WSEndpointCollectionBasedMOMListener listener;
    private final WeakHashMap<WSEndpoint, SequenceManager> sequenceManagersForDeferredRegistration = new WeakHashMap<WSEndpoint, SequenceManager>();
        
    private SequenceManagerFactory() {
        // TODO: load from external configuration and revert to default if not present

        listener = new WSEndpointCollectionBasedMOMListener(this, SequenceManager.MANAGED_BEAN_NAME, sequenceManagersForDeferredRegistration);
        listener.initialize();
    }

    /**
     * Creates new {@link SequenceManager} instance. This operation should be called only once per endpoint and/or endpoint client.
     *
     * @param persistent specifies whether returned {@link SequenceManager} instance should support persistent message storage
     * @param uniqueEndpointId unique identifier of the WS endpoint for which this particular sequence manager will be used. The endpoint
     * identifier must be different for the client and for the server side.
     * @param inboundQueueBuilder delivery queue builder that will be used to create delivery queue for all newly created inbound sequences
     * @param outboundQueueBuilder delivery queue builder that will be used to create delivery queue for all newly created outbound sequences
     * @param managedObjectManager object manager managing the newly created {@link SequenceManager} instance
     * @return newly created {@link SequenceManager} instance
     */
    public SequenceManager createSequenceManager(boolean persistent, String uniqueEndpointId, DeliveryQueueBuilder inboundQueueBuilder, DeliveryQueueBuilder outboundQueueBuilder, RmConfiguration configuration, Container container, LocalIDManager localIDManager) {
        synchronized (INSTANCE) {
            SequenceManager result;
            if (persistent) {
                result = new PersistentSequenceManager(uniqueEndpointId, inboundQueueBuilder, outboundQueueBuilder, configuration, container, localIDManager);
            } else {
                result = new InVmSequenceManager(uniqueEndpointId, inboundQueueBuilder, outboundQueueBuilder, configuration, container, localIDManager);
            }
            
            ManagedObjectManager mom = configuration.getManagedObjectManager();
            handleMOMRegistration(result, mom, true);

            return result;
        }
    }
    
    /**
     * Disposes the sequence manager properly and unregisters it from the ManagedOnjectManager
     * 
     * @param manager {@link SequenceManager} instance to be disposed
     */
    public void dispose(SequenceManager manager, RmConfiguration configuration) {
        synchronized (INSTANCE) {
            manager.dispose();

            ManagedObjectManager mom = configuration.getManagedObjectManager();
            handleMOMRegistration(manager, mom, false);
        }
    }

    /**
     * Handles (un)registration process of {@code SequenceManager} in {@link ManagedObjectManager}.
     * 
     * @param manager SequenceManager to be (un)registered
     * @param managedObjectManager ManagedObjectManager instance where SequenceManager will be registered at
     * @param register {@code true} if the manager should be registered, {@code false} for unregistration
     */
    private void handleMOMRegistration(final SequenceManager manager, final ManagedObjectManager managedObjectManager, final boolean register) {
        if (manager == null || managedObjectManager == null) {
            return;
        }

        if (!listener.canRegisterAtMOM() && (managedObjectManager instanceof WSEndpointMOMProxy)) {
            // SequenceManager cannot be (un)registered directly so postpone its (un)registration until JMX connection
            // is created
            final WSEndpointMOMProxy endpointMOMProxy = (WSEndpointMOMProxy) managedObjectManager;
            final WSEndpointImpl wsEndpoint = endpointMOMProxy.getWsEndpoint();

            if (register) {
                sequenceManagersForDeferredRegistration.put(wsEndpoint, manager);
            } else {
                sequenceManagersForDeferredRegistration.remove(wsEndpoint);
            }
        } else {
            // 1) SequenceManager can be (un)registered directly - process its (un)registration
            // 2) (not expected) SequenceManager cannot be (un)registered directly but also it cannot be postponed hence
            //    the (un)registration is going to be forced (managedObjectManager (un)registration as well)

            if (register) {
                listener.registerAtMOM(manager, managedObjectManager);
            } else {
                listener.unregisterFromMOM(manager, managedObjectManager);
            }
        }
    }
    
}
